/*
 * CObservationDownsampler.h
 *
 *  Created on: 2 Mar 2020
 *      Author: idelpino
 */

#ifndef COBSERVATIONDOWNSAMPLER_H_
#define COBSERVATIONDOWNSAMPLER_H_

#include "structs_definitions.h"
#include "math.h"
#include <numeric>
#include <vector>
#include <pcl/impl/point_types.hpp>
#include <pcl/point_cloud.h>

#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include <boost/accumulators/statistics/mean.hpp>

using namespace boost::accumulators;

class CObservationDownsampler
{
private:
  float angular_resolution_;
  float propagation_additive_noise_;
  float propagation_z_additive_noise_;
  std::vector<std::vector<float>> centroids_;
  std::vector<std::vector<pcl::PointXYZRGBNormal>> observations_;
  std::vector<int> num_points_;
  std::vector<accumulator_set<float, stats<tag::mean, tag::variance> > > acc_;
public:
  CObservationDownsampler(float angular_resolution, float propagation_additive_noise, float propagation_z_additive_noise);
  virtual ~CObservationDownsampler();

  void addGroundObservationInSensorFrame(const pcl::PointXYZRGBNormal reference_in_sensor_frame,
                                         const pcl::PointXYZRGBNormal ground_observation_in_sensor_frame);

  void generateNewGroundReferences(const int reference_index, const pcl::PointXYZRGBNormal& reference,
                                   pcl::PointCloud<pcl::PointXYZRGBNormal>::Ptr ground_reference_cloud_ptr,
                                   std::vector<Eigen::Matrix3d> & references_covariance_matrix_vector,
                                   std::vector<std::vector<int>>& edges);

  void predictZWithVariance(const pcl::PointXYZRGBNormal reference, const float x, const float y, float& z_predicted,
                            float& var_z_predicted);
};

#endif /* COBSERVATIONDOWNSAMPLER_H_ */
