/*
 * time_supervisor.h
 *
 *  Created on: 3 Dec 2019
 *      Author: idelpino
 */

#ifndef _TIME_SUPERVISOR_H_
#define _TIME_SUPERVISOR_H_

#include <string>
#include <iostream>
#include <list>
#include <memory>
#include <ros/ros.h>

struct StPerformanceStatistics
{
  double last_time_;
  double mean_time_;
  double max_time_;
  double min_time_;
  double total_time_;
  int iterations_;
};

class CFunctionPerformance
{
private:
  StPerformanceStatistics statistics_;
  std::string function_name_;
public:
  CFunctionPerformance();
  ~CFunctionPerformance();

  void addData(double time_expended);
  const StPerformanceStatistics getStatistics(void);

  const std::string getFunctionName(void)
  {
    return (function_name_);
  }

  void setFunctionName(const std::string &functionName)
  {
    function_name_ = functionName;
  }
};

class CTimeSupervisor
{
private:
  std::list<CFunctionPerformance> function_list_;
public:
  CTimeSupervisor();
  ~CTimeSupervisor();
  void addData(std::string function_name, double time_expended);
  void printStatistics();
  typedef boost::shared_ptr<CTimeSupervisor> Ptr;
};

class CFunctionMonitor
{
private:
  CTimeSupervisor::Ptr my_supervisor_;
  std::string function_name_;
  double initial_time_;
  double final_time_;
  double time_expended_;
public:
  CFunctionMonitor(const std::string function_name, const CTimeSupervisor::Ptr time_supervisor_ptr);
  ~CFunctionMonitor();
};

#endif /* _TIME_SUPERVISOR_H_ */
