#include "shallow_neural_net.h"

ShallowNeuralNet::ShallowNeuralNet(void)
{

}

ShallowNeuralNet::~ShallowNeuralNet(void)
{

}

double ShallowNeuralNet::predict(Eigen::VectorXd features)
{
  int predicted_class = -1;

  assert(features.rows() == this->number_of_features_ && "Error using ShallowNeuralNet::predict, number of features not expected!");

  //std::cout << "features = " << std::endl << features << std::endl;
  Eigen::VectorXd x_offset_applied(number_of_features_, 1);
  x_offset_applied = features - this->x_offset_;

  //std::cout << "x_offset_applied = " << std::endl << x_offset_applied << std::endl;

  Eigen::VectorXd x_gain_plus_y_min(number_of_features_, 1);
  for (int i = 0; i < number_of_features_; ++i)
    x_gain_plus_y_min(i,0) = x_offset_applied(i, 0) * this->gain_(i, 0) + this->y_min_(0, 0);

  //std::cout << "x_gain_plus_y_min = " << std::endl << x_gain_plus_y_min << std::endl;

  Eigen::VectorXd z1(this->number_of_neurons_in_hidden_layer_, 1);
  z1 = this->b1_ + (this->w1_ * x_gain_plus_y_min);

  //std::cout << "z1 = " << std::endl << z1 << std::endl;

  Eigen::VectorXd a1(this->number_of_neurons_in_hidden_layer_, 1);
  for (int i = 0; i < this->number_of_neurons_in_hidden_layer_; ++i)
    a1(i,0) = (2.0 / (1.0 + exp(-2.0 * z1(i,0)))) - 1.0;

  //std::cout << "a1 = " << std::endl << a1 << std::endl;

  Eigen::VectorXd z2(this->number_of_neurons_in_output_layer_, 1);
  z2 = this->b2_ + (this->w2_ * a1);
  //std::cout << "z2 = " << std::endl << z2 << std::endl;

  double z2_max = z2.maxCoeff();
  //std::cout << "z2_max = " << std::endl << z2_max << std::endl;
  z2.array() -= z2_max;
  //std::cout << "z2 = " << std::endl << z2 << std::endl;

  Eigen::VectorXd numerator(this->number_of_neurons_in_output_layer_, 1);
  for(int i = 0; i < this->number_of_neurons_in_output_layer_; ++i)
    numerator(i,0) = exp(z2(i,0));

  //std::cout << "numerator = " << std::endl << numerator << std::endl;

  double denominator = numerator.sum();
  //std::cout << "denominator = " << std::endl << denominator << std::endl;

  Eigen::VectorXd prediction(this->number_of_neurons_in_output_layer_, 1);
  prediction = numerator / denominator;

  //std::cout << "prediction = " << std::endl << prediction << std::endl;

  int index_of_max_prob = -1;
  double max_prob = -1.0;
  for (int i = 0; i < this->number_of_neurons_in_output_layer_; ++i)
  {
    if(prediction(i,0) > max_prob)
    {
      max_prob = prediction(i,0);
      index_of_max_prob = i;
    }
  }
  predicted_class = index_of_max_prob;

  switch(predicted_class)
  {
    case 0:
      predicted_class = CLASS_OBSTACLE;
      break;
    case 1:
      predicted_class = ROAD;
      break;
    case 2:
      predicted_class = SIDEWALK;
      break;
    case 3:
      predicted_class = TERRAIN;
      break;
    case 4:
      predicted_class = VEGETATION;
      break;
    default:
      assert(false && "ERROR in ShallowNeuralNet::predict: unexpected predicted class!!");
  }

  //std::cout << "predicted_class = " << std::endl << predicted_class << std::endl;

  return(predicted_class);
}
