/*
 * time_supervisor.cpp
 *
 *  Created on: 3 Dec 2019
 *      Author: idelpino
 */
#include "time_supervisor.h"

CFunctionPerformance::CFunctionPerformance()
{
  statistics_.last_time_ = 0.0;

  statistics_.mean_time_ = 0.0;
  statistics_.max_time_ = 0.0;
  statistics_.min_time_ = 1000.0;

  statistics_.total_time_ = 0.0;

  statistics_.iterations_ = 0;
  function_name_ = "unitialized";
}

CFunctionPerformance::~CFunctionPerformance()
{
}

void CFunctionPerformance::addData(double time_expended)
{
  statistics_.last_time_ = time_expended;
  //if (time_expended > 0.00001) // to avoid taking into account empty processes
  //{
    statistics_.iterations_++;
    statistics_.total_time_ += time_expended;
    statistics_.mean_time_ = statistics_.total_time_ / (double)statistics_.iterations_;

    if (time_expended < statistics_.min_time_)
      statistics_.min_time_ = time_expended;

    if (time_expended > statistics_.max_time_)
      statistics_.max_time_ = time_expended;
  //}
}

const StPerformanceStatistics CFunctionPerformance::getStatistics(void)
{
  return (statistics_);
}

/////////////////// Time supervisor /////////////////////////////////////////////

CTimeSupervisor::CTimeSupervisor()
{
}

CTimeSupervisor::~CTimeSupervisor()
{
}

void CTimeSupervisor::addData(std::string function_name, double time_expended)
{
  bool function_found = false;
  for (std::list<CFunctionPerformance>::iterator funct = function_list_.begin(); funct != function_list_.end(); ++funct)
  {
    if (funct->getFunctionName() == function_name)
    {
      function_found = true;
      funct->addData(time_expended);
    }
  }

  if (!function_found)
  {
    CFunctionPerformance new_function;
    new_function.setFunctionName(function_name);
    new_function.addData(time_expended);
    function_list_.push_back(new_function);
  }

}

void CTimeSupervisor::printStatistics()
{
  StPerformanceStatistics statistics;
  double complete_total = 0.0;
  double complete_mean  = 0.0;

  for (std::list<CFunctionPerformance>::iterator funct = function_list_.begin(); funct != function_list_.end(); ++funct)
  {
    statistics = funct->getStatistics();
    std::cout << "Function: " << funct->getFunctionName() << "    Time statistics: " << std::endl;
    std::cout << "last = " << statistics.last_time_ << "    mean = " << statistics.mean_time_ << "    min = "
        << statistics.min_time_ << "   max= " << statistics.max_time_ << "    total = " << statistics.total_time_
        << "    iterations = " << statistics.iterations_ << std::endl << std::endl;

    complete_total += statistics.total_time_;
  }

  double check = 0.0;
  for (std::list<CFunctionPerformance>::iterator funct = function_list_.begin(); funct != function_list_.end(); ++funct)
  {
    statistics = funct->getStatistics();
    double current_function_percentage = 100.0 * statistics.total_time_ / complete_total;
    std::cout << "Function: " << funct->getFunctionName() << "    Load percentage =  "
        << current_function_percentage << std::endl;

    check += current_function_percentage;
  }
  // TODO: Debug time supervisor, or find another method to monitorize function times, for now don't use this feature!
  //assert(check > 99.90 && check < 100.01 && "Error in printStatistics: percentage greater than 100.0 !!");

  std::cout << "Total processing time : " << complete_total << std::endl;
  std::cout << "Mean processing time : " << complete_total / statistics.iterations_ << std::endl;
}

std::string function_name_;
double initial_time_;
double final_time_;
double time_expended_;

CFunctionMonitor::CFunctionMonitor(const std::string function_name, const CTimeSupervisor::Ptr time_supervisor_ptr)
{
  my_supervisor_ = time_supervisor_ptr;
  function_name_ = function_name;
  initial_time_ = ros::WallTime::now().toSec();
  final_time_ = 0.0;
  time_expended_ = 0.0;
}

CFunctionMonitor::~CFunctionMonitor()
{
  final_time_ = ros::WallTime::now().toSec();
  time_expended_ = final_time_ - initial_time_;
  //assert(time_expended_ >= 0.0 && "Error in CFunctionMonitor: time negative!!");
  if(time_expended_ < 0.0)
    std::cout << "WARNING in CFunctionMonitor: time negative!!" << std::endl;
  else
    my_supervisor_->addData(function_name_, time_expended_);
}
